@@ -56,7 +56,7 @@ def login(port, verbose):
         else:
             click.echo('Already logged in.')
         context.EnsureConnected()
-        gubun = context.KOA_Functions('GetServerGubun', '')
+        gubun = context.GetServerGubun()
         if gubun == '1':
             click.echo('Logged into Simulation server.')
         else:
@@ -74,22 +74,22 @@ def autologin(port, verbose):
     from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout) as context:
         context.EnsureConnected()
-        context.KOA_Functions('ShowAccountWindow', '')
+        context.ShowAccountWindow()
 
 @cli.group(context_settings=CONTEXT_SETTINGS, short_help='Update openapi metadata.')
 def update():
     pass
 
-@update.command(context_settings=CONTEXT_SETTINGS, short_help='Update openapi metadata.')
+@update.command(context_settings=CONTEXT_SETTINGS, short_help='Update openapi TR metadata.')
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
-def trinfo(verbose):
+def trdata(verbose):
     set_verbosity(verbose)
     from koapy.openapi.TrInfo import TrInfo
     TrInfo.dump_trinfo_by_code()
 
-@update.command(context_settings=CONTEXT_SETTINGS, short_help='Update openapi metadata.')
+@update.command(context_settings=CONTEXT_SETTINGS, short_help='Update openapi realtype metadata.')
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
-def realtype(verbose):
+def realdata(verbose):
     set_verbosity(verbose)
     from koapy.openapi.RealType import RealType
     RealType.dump_realtype_by_desc()
@@ -589,7 +589,7 @@ def holidays(output, verbose):
         df = pd.DataFrame.from_records(data, columns=columns)
         click.echo(df.to_markdown())
     else:
-        from koapy.utils.krx.closing import download_holidays_as_excel
+        from koapy.utils.krx.holiday import download_holidays_as_excel
         if output and not output.endswith('.xls'):
             output += '.xls'
         download_holidays_as_excel(output)
@@ -624,7 +624,7 @@ def userinfo(port, verbose):
         }.get(context.GetLoginInfo('FIREW_SECGB'), '알수없음')
         result['접속서버 구분'] = {
             '1': '모의투자',
-        }.get(context.GetLoginInfo('GetServerGubun'), '실서버')
+        }.get(context.GetServerGubun(), '실서버')
 
         click.echo(pd.Series(result).to_markdown())
 
@@ -633,23 +633,21 @@ def userinfo(port, verbose):
 @click.option('-p', '--port', metavar='PORT', help='Port number of grpc server (optional).')
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
 def deposit(account, port, verbose):
-    if account is None:
-        logging.info('Account not given. Using first account available.')
-        # fail_with_usage()
-
     set_verbosity(verbose)
 
-    import pandas as pd
+    if account is None:
+        logging.info('Account not given. Using first account available.')
+
     from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
 
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
         context.EnsureConnected()
 
         if account is None:
-            account = context.GetAccounts()[0]
+            account = context.GetAccountList()[0]
 
         result = context.GetDepositInfo(account)
-        click.echo(pd.Series(result).to_markdown())
+        click.echo(result.to_markdown())
 
 @get.command(context_settings=CONTEXT_SETTINGS, short_help='Get account evaluation.')
 @click.option('-a', '--account', metavar='ACCNO', help='Account number.')
@@ -660,11 +658,10 @@ def deposit(account, port, verbose):
 @click.option('-p', '--port', metavar='PORT', help='Port number of grpc server (optional).')
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
 def evaluation(account, include_delisted, exclude_delisted, for_each, as_summary, port, verbose):
+    set_verbosity(verbose)
+
     if account is None:
         logging.info('Account not given. Using first account available.')
-        # fail_with_usage()
-
-    set_verbosity(verbose)
 
     if exclude_delisted:
         include_delisted = False
@@ -672,17 +669,16 @@ def evaluation(account, include_delisted, exclude_delisted, for_each, as_summary
     if as_summary:
         for_each = False
         lookup_type = '1'
-    else:
+    elif for_each:
         lookup_type = '2'
 
-    import pandas as pd
     from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
 
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
         context.EnsureConnected()
 
         if account is None:
-            account = context.GetAccounts()[0]
+            account = context.GetAccountList()[0]
 
         single, multi = context.GetAccountEvaluationStatusAsSeriesAndDataFrame(account, include_delisted)
         click.echo('[계좌평가현황요청] : [계좌평가현황]')
@@ -714,13 +710,11 @@ def evaluation(account, include_delisted, exclude_delisted, for_each, as_summary
 @click.option('-p', '--port', metavar='PORT', help='Port number of grpc server (optional).')
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
 def orders(account, date, reverse, executed_only, not_executed_only, stock_only, bond_only, sell_only, buy_only, code, starting_order_no, port, verbose):
-    if account is None:
-        logging.info('Account not given. Using first account available.')
-        # fail_with_usage()
-
     set_verbosity(verbose)
 
-    import pandas as pd
+    if account is None:
+        logging.info('Account not given. Using first account available.')
+
     from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
 
     sort_type = '1'
@@ -745,10 +739,28 @@ def orders(account, date, reverse, executed_only, not_executed_only, stock_only,
         context.EnsureConnected()
 
         if account is None:
-            account = context.GetAccounts()[0]
+            account = context.GetAccountList()[0]
 
         click.echo(context.GetOrderLogAsDataFrame3(account, date, sort_type, asset_type, order_type, code, starting_order_no).to_markdown())
 
+@get.command(context_settings=CONTEXT_SETTINGS, short_help='Get OpenApi module installation path.')
+@click.option('-p', '--port', metavar='PORT', help='Port number of grpc server (optional).')
+@click.option('-v', '--verbose', count=True, help='Verbosity.')
+def modulepath(port, verbose):
+    set_verbosity(verbose)
+    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
+        click.echo(context.GetAPIModulePath())
+
+@get.command(context_settings=CONTEXT_SETTINGS, short_help='Get error message for error code.')
+@click.option('-e', '--err-code', metavar='ERR', type=int, help='Error code to check.')
+@click.option('-v', '--verbose', count=True, help='Verbosity.')
+def errmsg(err_code, verbose):
+    set_verbosity(verbose)
+    from koapy.openapi.KiwoomOpenApiError import KiwoomOpenApiError
+    err_msg = KiwoomOpenApiError.get_error_message_by_code(err_code)
+    click.echo('[%d] %s' % (err_code, err_msg))
+
 @cli.command(context_settings=CONTEXT_SETTINGS, short_help='Watch realtime data.')
 @click.option('-c', '--code', 'codes', metavar='CODE', multiple=True, help='Stock code to get. Can set multiple times.')
 @click.option('-i', '--input', metavar='FILENAME', type=click.Path(), help='Text or excel file containing codes. Alternative to --codes option.')
@@ -892,14 +904,19 @@ def order(request_name, screen_no, account_no, order_type, code, quantity, price
 
     set_verbosity(verbose)
 
+    import pprint
+
     from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
     from google.protobuf.json_format import MessageToDict
 
+    pp = pprint.PrettyPrinter()
+
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
         context.EnsureConnected()
 
-        for response in context.OrderCall(self, request_name, screen_no, account_no, order_type, code, quantity, price, quote_type, original_order_no):
-            click.echo(MessageToDict(response)) # TODO: more pretty print
+        for response in context.OrderCall(request_name, screen_no, account_no, order_type, code, quantity, price, quote_type, original_order_no):
+            dic = MessageToDict(response)
+            click.echo(pp.pformat(dic))
 
 if __name__ == '__main__':
     cli()
