@@ -9,6 +9,7 @@ from koapy.openapi.TrInfo import TrInfo
 from koapy.openapi.RealType import RealType
 
 from koapy.utils.notimplemented import isimplemented
+from koapy.utils.itertools import chunk
 
 class KiwoomOpenApiLoggingEventHandler(BaseKiwoomOpenApiEventHandler):
 
@@ -194,6 +195,9 @@ class KiwoomOpenApiLoginEventHandler(BaseKiwoomOpenApiEventHandler):
         super().__init__(control)
         self._request = request
 
+    def on_enter(self):
+        KiwoomOpenApiError.try_or_raise(self.control.CommConnect())
+
     def OnEventConnect(self, errcode):
         if errcode < 0:
             error = KiwoomOpenApiError(errcode)
@@ -206,9 +210,10 @@ class KiwoomOpenApiLoginEventHandler(BaseKiwoomOpenApiEventHandler):
 
 class KiwoomOpenApiTrEventHandler(BaseKiwoomOpenApiEventHandler):
 
-    def __init__(self, control, request):
+    def __init__(self, control, request, screen_manager):
         super().__init__(control)
         self._request = request
+        self._screen_manager = screen_manager
 
         self._rqname = request.request_name
         self._trcode = request.transaction_code
@@ -250,6 +255,17 @@ class KiwoomOpenApiTrEventHandler(BaseKiwoomOpenApiEventHandler):
 
         self._is_stop_condition = is_stop_condition
 
+    def on_enter(self):
+        self._scrnno = self._screen_manager.borrow_screen(self._scrnno)
+        self.add_callback(self._screen_manager.return_screen, self._scrnno)
+        for k, v in self._inputs.items():
+            self.control.SetInputValue(k, v)
+        KiwoomOpenApiError.try_or_raise(
+            self.control.RateLimitedCommRqData(self._rqname, self._trcode, 0, self._scrnno, self._inputs))
+
+    def on_exit(self):
+        self.control.DisconnectRealData(self._scrnno)
+
     def OnReceiveTrData(self, scrnno, rqname, trcode, recordname, prevnext, datalength, errorcode, message, splmmsg):
         if (rqname, trcode, scrnno) == (self._rqname, self._trcode, self._scrnno):
             response = KiwoomOpenApiService_pb2.CustomCallAndListenResponse()
@@ -304,9 +320,10 @@ class KiwoomOpenApiTrEventHandler(BaseKiwoomOpenApiEventHandler):
 
 class KiwoomOpenApiOrderEventHandler(BaseKiwoomOpenApiEventHandler):
 
-    def __init__(self, control, request):
+    def __init__(self, control, request, screen_manager):
         super().__init__(control)
         self._request = request
+        self._screen_manager = screen_manager
 
         self._rqname = request.request_name
         self._scrnno = request.screen_no
@@ -327,6 +344,21 @@ class KiwoomOpenApiOrderEventHandler(BaseKiwoomOpenApiEventHandler):
         self._is_stop_condition = lambda row: False
         self._inputs = {}
 
+    def on_enter(self):
+        self._scrnno = self._screen_manager.borrow_screen(self._scrnno)
+        self.add_callback(self._screen_manager.return_screen, self._scrnno)
+        KiwoomOpenApiError.try_or_raise(
+            self.control.SendOrder(
+                self._rqname,
+                self._scrnno,
+                self._accno,
+                self._ordertype,
+                self._code,
+                self._qty,
+                self._price,
+                self._hogagb,
+                self._orgorderno))
+
     def OnReceiveMsg(self, scrnno, rqname, trcode, msg):
         if (rqname, scrnno) == (self._rqname, self._scrnno):
             response = KiwoomOpenApiService_pb2.CustomCallAndListenResponse()
@@ -471,9 +503,13 @@ class KiwoomOpenApiOrderEventHandler(BaseKiwoomOpenApiEventHandler):
 
 class KiwoomOpenApiRealEventHandler(BaseKiwoomOpenApiEventHandler):
 
-    def __init__(self, control, request):
+    _num_codes_per_screen = 100
+    _default_real_type = '0'
+
+    def __init__(self, control, request, screen_manager):
         super().__init__(control)
         self._request = request
+        self._screen_manager = screen_manager
 
         self._screen_no = request.screen_no
         self._code_list = request.code_list
@@ -484,6 +520,27 @@ class KiwoomOpenApiRealEventHandler(BaseKiwoomOpenApiEventHandler):
         self._readable_names = request.flags.readable_names
         self._fast_parse = request.flags.fast_parse
 
+        self._code_lists = [';'.join(codes) for codes in chunk(self._code_list, self._num_codes_per_screen)]
+        if len(self._screen_no) == 0:
+            self._screen_nos = [None for i in range(len(self._code_lists))]
+        elif len(self._screen_no) < len(self._code_lists):
+            logging.warning('Given screen nos are not sufficient.')
+            self._screen_nos = list(self._screen_nos) + [None for i in range(len(self._code_lists) - len(self.screen_nos))]
+        else:
+            self._screen_nos = self._screen_no
+        self._fid_list_joined = ';'.join([str(fid) for fid in self._fid_list])
+        self._real_type_explicit = self._real_type or self._default_real_type
+
+    def on_enter(self):
+        for screen_no, code_list in zip(self._screen_nos, self._code_lists):
+            screen_no = self._screen_manager.borrow_screen(screen_no)
+            self.add_callback(self._screen_manager.return_screen, screen_no)
+            self.add_callback(self.control.DisconnectRealData, screen_no)
+            self.add_callback(self.control.SetRealRemove, screen_no, code_list)
+
+            KiwoomOpenApiError.try_or_raise(
+                self.control.SetRealReg(screen_no, code_list, self._fid_list_joined, self._real_type_explicit))
+
     def OnReceiveRealData(self, code, realtype, realdata):
         if code in self._code_list:
             response = KiwoomOpenApiService_pb2.CustomCallAndListenResponse()
