@@ -6,7 +6,8 @@ import signal
 import contextlib
 
 from PyQt5.QtWidgets import QApplication, QSystemTrayIcon, QMenu, QStyle
-from PyQt5.QtCore import QTimer, QObject, pyqtSignal
+from PyQt5.QtCore import QTimer, QObject, QUrl, pyqtSignal
+from PyQt5.QtGui import QDesktopServices
 
 from koapy.grpc.KiwoomOpenApiServiceServer import KiwoomOpenApiServiceServer
 from koapy.openapi.KiwoomOpenApiError import KiwoomOpenApiError
@@ -32,7 +33,7 @@ class KiwoomOpenApiTrayApplication(QObject):
 
         set_verbosity(self._verbose)
 
-        self._app = QApplication(args[:1] + remaining_args)
+        self._app = QApplication(remaining_args)
         self._control = KiwoomOpenApiQAxWidget()
         self._server = KiwoomOpenApiServiceServer(self._control, port=self._port)
 
@@ -56,6 +57,13 @@ class KiwoomOpenApiTrayApplication(QObject):
         self._connectionStatusAction.setEnabled(False)
         self._serverStatusAction = menu.addAction('Server: Unknown')
         self._serverStatusAction.setEnabled(False)
+        menu.addSection('Links')
+        self._openApiUrl = "https://www3.kiwoom.com/nkw.templateFrameSet.do?m=m1408000000"
+        self._qnaUrl = "https://bbn.kiwoom.com/bbn.openAPIQnaBbsList.do"
+        openApiAction = menu.addAction('Kiwoom OpenAPI+')
+        openApiAction.triggered.connect(self._openOpenApi)
+        qnaAction = menu.addAction('Kiwoom OpenAPI+ QNA')
+        qnaAction.triggered.connect(self._openQna)
         menu.addSection('Exit')
         exitAction = menu.addAction('Exit')
         exitAction.triggered.connect(self._exit)
@@ -109,7 +117,7 @@ class KiwoomOpenApiTrayApplication(QObject):
             state = self._control.GetConnectState()
             if state == 1:
                 self._connectionStatusAction.setText('Status: Connected')
-                server = self._control.GetLoginInfo('GetServerGubun')
+                server = self._control.GetServerGubun()
                 if server == '1':
                     self._serverStatusAction.setText('Server: Simulation')
                 else:
@@ -161,11 +169,19 @@ class KiwoomOpenApiTrayApplication(QObject):
         self._ensureConnectedAndThen()
 
     def _showAccountWindow(self):
-        self._control.koapy_Functions('ShowAccountWindow', '')
+        self._control.ShowAccountWindow()
 
     def _configureAutoLogin(self):
         self._ensureConnectedAndThen(self._showAccountWindow)
 
+    def _openOpenApi(self):
+        url = QUrl(self._openApiUrl)
+        QDesktopServices.openUrl(url)
+
+    def _openQna(self):
+        url = QUrl(self._qnaUrl)
+        QDesktopServices.openUrl(url)
+
     def _onInterrupt(self, signum, _frame):
         self._exit(signum + 100)
 
@@ -222,6 +238,22 @@ class KiwoomOpenApiTrayApplication(QObject):
     def _exitForRestart(self):
         return self._exit(self._should_restart_exit_code)
 
+    def __getattr__(self, name):
+        return self._app
+
+    @property
+    def control(self):
+        return self._control
+
+    def get_control(self):
+        return self._control
+
+    def exec_(self):
+        return self._exec()
+
+    def exit(self, return_code=0):
+        return self._exit(return_code)
+
     def execAndExit(self):
         code = self._exec()
         sys.exit(code)
